// services/user.ts
import { ensureStringId } from '@/utils/db-helper'
import { Result, createSuccess, createError } from '@/types/result'
import { cloudApi } from '@/api'
import { type UserInfo, UserRole } from '@/types/user'
import { USE_MOCK } from '@/services'
import Taro from '@tarojs/taro'

// Mock用户数据 - 扩展以包含更多字段
const createMockUser = () => ({
  nickname: '测试用户',
  avatar: 'https://images.unsplash.com/photo-1582845512747-e42001c95638?ixlib=rb-1.2.1&auto=format&fit=crop&w=400&h=100&q=80',
  openid: 'mock_openid_' + Date.now().toString().slice(-6),
  role: UserRole.USER,
  isRegistered: true,
  points: 100,
  exp: 50,
  level: 2,
  gameCount: 5,
  roomCount: 3
})

export const userService = {
  // 统一登录方法 - 新增方法处理mock/真实环境
  async login(): Promise<Result<UserInfo>> {
    // 判断是否为mock环境
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess(createMockUser()))
        }, 300)
      })
    }
    
    // 真实环境
    try {
      const loginRes = await Taro.login()
      if (!loginRes.code) {
        return createError('登录失败')
      }
      
      const result = await this.getOpenId(loginRes.code)
      if (!result.success || !result.data) {
        return createError(result.message || '获取OpenID失败')
      }
      
      // 返回包含openid的用户信息
      return createSuccess({
        openid: result.data.openid,
        nickname: '',
        avatar: '',
        role: UserRole.USER
      })
    } catch (error) {
      console.error('登录失败:', error)
      return createError(error.message || '登录失败')
    }
  },
  
  // 统一注册方法 - 新增方法处理mock/真实环境
  async register(openid: string): Promise<Result<UserInfo>> {
    // 判断是否为mock环境
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess(createMockUser()))
        }, 300)
      })
    }
    
    // 真实环境
    try {
      const profileRes = await Taro.getUserProfile({
        desc: '用于完善用户资料'
      })
      
      if (!profileRes.userInfo) {
        return createError('获取用户信息失败')
      }
      
      // 构建用户信息
      const userData: UserInfo = {
        openid,
        nickname: profileRes.userInfo.nickName,
        avatar: profileRes.userInfo.avatarUrl,
        role: UserRole.USER
      }
      
      // 保存到云端
      await this.saveUserToCloud(userData)
      
      // 返回完整的用户信息
      return createSuccess({
        ...userData,
        isRegistered: true
      })
    } catch (error) {
      console.error('注册失败:', error)
      return createError(error.message || '注册失败')
    }
  },
  
  // 获取用户信息
  async getUserInfo(): Promise<Result<UserInfo>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess(createMockUser()))
        }, 300)
      })
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<UserInfo>('getUserInfo')
      .then(result => {
        // 如果没有返回数据，使用mock数据
        if (result.success && !result.data) {
          return createSuccess(createMockUser())
        }
        return result
      })
      .catch(error => {
        console.error('获取用户信息失败:', error)
        return createError(error.message || '获取用户信息失败')
      })
  },
  
  // 获取OpenID
  async getOpenId(code: string): Promise<Result<{ openid: string, session_key?: string, unionid?: string }>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return createSuccess({ 
        openid: 'mock_openid_' + Date.now().toString().slice(-6)
      })
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ openid: string, session_key?: string, unionid?: string, errcode?: number, errmsg?: string }>('getOpenId', { code })
      .then(result => {
        // 检查响应中是否包含错误码
        if (result.success && result.data && result.data.errcode && result.data.errcode !== 0) {
          return createError(result.data.errmsg || '获取OpenID失败')
        }
        return result
      })
      .catch(error => {
        console.error('获取OpenID失败:', error)
        return createError(error.message || '获取OpenID失败')
      })
  },
  
  // 保存用户信息到云端
  async saveUserToCloud(userData: UserInfo): Promise<Result<string>> {
    // 如果在开发环境或非小程序环境，返回Mock成功
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return createSuccess('mock_doc_id')
    }
    
    try {
      const db = cloudApi.db()
      
      // 查询用户是否存在
      const userQuery = await db.collection('users').where({
        openid: userData.openid
      }).get()
      
      // 如果用户已存在，更新信息
      if (userQuery.data && userQuery.data.length > 0) {
        const docId = ensureStringId(userQuery.data[0]._id)
        
        await db.collection('users').doc(docId).update({
          data: {
            nickname: userData.nickname,
            avatar: userData.avatar,
            role: userData.role,
            updateTime: db.serverDate()
          }
        })
        return createSuccess(docId)
      } 
      
      // 创建新用户
      const addResult = await db.collection('users').add({
        data: {
          ...userData,
          createTime: db.serverDate(),
          updateTime: db.serverDate()
        }
      })
      
      // 确保返回的 _id 是字符串
      const newId = typeof addResult._id === 'number' 
        ? String(addResult._id) 
        : addResult._id as string
        
      return createSuccess(newId)
    } catch (error) {
      console.error('保存用户信息到云端失败:', error)
      return createError(error.message || '保存用户信息失败')
    }
  },
  
  // 获取用户角色(用于权限控制)
  async getUserRole(openid?: string): Promise<Result<string>> {
    // 如果在开发环境或非小程序环境，返回Mock角色
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return createSuccess('user')
    }
    
    try {
      // 如果没有提供openid，获取当前用户
      if (!openid) {
        const userResult = await this.getUserInfo()
        if (!userResult.success || !userResult.data) {
          return createError('无法获取当前用户信息')
        }
        openid = userResult.data.openid
      }
      
      const db = cloudApi.db()
      
      // 查询用户记录
      const userQuery = await db.collection('users').where({
        openid
      }).get()
      
      if (userQuery.data && userQuery.data.length > 0) {
        return createSuccess(userQuery.data[0].role || 'user')
      }
      
      // 用户不存在，返回默认角色
      return createSuccess('user')
    } catch (error) {
      console.error('获取用户角色失败:', error)
      return createError(error.message || '获取用户角色失败')
    }
  }
}